from __future__ import unicode_literals

import re
import pytz
import logging

from croniter.croniter import croniter
from datetime import datetime
from django.apps import apps
from django.conf import settings
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import PermissionsMixin, UserManager
from django.core import validators
from django.core.exceptions import ValidationError
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone as django_timezone
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.lru_cache import lru_cache
from django.utils.translation import ugettext_lazy as _
from django_fsm import transition, FSMIntegerField
from model_utils import FieldTracker
from reversion import revisions as reversion
from reversion.models import Version

from waldur_core.core.fields import CronScheduleField, UUIDField
from waldur_core.core.validators import validate_name, MinCronValueValidator
from waldur_core.logging.loggers import LoggableMixin


logger = logging.getLogger(__name__)


class DescribableMixin(models.Model):
    """
    Mixin to add a standardized "description" field.
    """
    class Meta(object):
        abstract = True

    description = models.CharField(_('description'), max_length=500, blank=True)


class NameMixin(models.Model):
    """
    Mixin to add a standardized "name" field.
    """

    class Meta(object):
        abstract = True

    name = models.CharField(_('name'), max_length=150, validators=[validate_name])


class UiDescribableMixin(DescribableMixin):
    """
    Mixin to add a standardized "description" and "icon url" fields.
    """
    class Meta(object):
        abstract = True

    icon_url = models.URLField(_('icon url'), blank=True)


class UuidMixin(models.Model):
    """
    Mixin to identify models by UUID.
    """
    class Meta(object):
        abstract = True

    uuid = UUIDField()


class ErrorMessageMixin(models.Model):
    """
    Mixin to add a standardized "error_message" field.
    """
    class Meta(object):
        abstract = True

    error_message = models.TextField(blank=True)


class CoordinatesMixin(models.Model):
    """
    Mixin to add a latitude and longitude fields
    """
    class Meta(object):
        abstract = True

    latitude = models.FloatField(null=True, blank=True)
    longitude = models.FloatField(null=True, blank=True)


class ScheduleMixin(models.Model):
    """
    Mixin to add a standardized "schedule" fields.
    """
    class Meta(object):
        abstract = True

    schedule = CronScheduleField(max_length=15, validators=[MinCronValueValidator(1)])
    next_trigger_at = models.DateTimeField(null=True)
    timezone = models.CharField(max_length=50, default=django_timezone.get_current_timezone_name)
    is_active = models.BooleanField(default=False)

    def update_next_trigger_at(self):
        base_time = django_timezone.now().replace(tzinfo=pytz.timezone(self.timezone))
        self.next_trigger_at = croniter(self.schedule, base_time).get_next(datetime)

    def save(self, *args, **kwargs):
        """
        Updates next_trigger_at field if:
         - instance become active
         - instance.schedule changed
         - instance is new
        """
        try:
            prev_instance = self.__class__.objects.get(pk=self.pk)
        except self.DoesNotExist:
            prev_instance = None

        if prev_instance is None or (not prev_instance.is_active and self.is_active or
                                     self.schedule != prev_instance.schedule):
            self.update_next_trigger_at()

        super(ScheduleMixin, self).save(*args, **kwargs)


@python_2_unicode_compatible
class User(LoggableMixin, UuidMixin, DescribableMixin, AbstractBaseUser, PermissionsMixin):
    username = models.CharField(
        _('username'), max_length=30, unique=True,
        help_text=_('Required. 30 characters or fewer. Letters, numbers and '
                    '@/./+/-/_ characters'),
        validators=[
            validators.RegexValidator(re.compile('^[\w.@+-]+$'), _('Enter a valid username.'), 'invalid')
        ])
    # Civil number is nullable on purpose, otherwise
    # it wouldn't be possible to put a unique constraint on it
    civil_number = models.CharField(_('civil number'), max_length=50, unique=True, blank=True, null=True, default=None)
    full_name = models.CharField(_('full name'), max_length=100, blank=True)
    native_name = models.CharField(_('native name'), max_length=100, blank=True)
    phone_number = models.CharField(_('phone number'), max_length=255, blank=True)
    organization = models.CharField(_('organization'), max_length=80, blank=True)
    organization_approved = models.BooleanField(_('organization approved'), default=False,
                                                help_text=_('Designates whether user organization was approved.'))
    job_title = models.CharField(_('job title'), max_length=40, blank=True)
    email = models.EmailField(_('email address'), max_length=75, blank=True)

    is_staff = models.BooleanField(_('staff status'), default=False,
                                   help_text=_('Designates whether the user can log into this admin '
                                               'site.'))
    is_active = models.BooleanField(_('active'), default=True,
                                    help_text=_('Designates whether this user should be treated as '
                                                'active. Unselect this instead of deleting accounts.'))
    is_support = models.BooleanField(_('support status'), default=False,
                                     help_text=_('Designates whether the user is a global support user.'))
    date_joined = models.DateTimeField(_('date joined'), default=django_timezone.now)
    registration_method = models.CharField(_('registration method'), max_length=50, default='default', blank=True,
                                           help_text=_('Indicates what registration method were used.'))
    agreement_date = models.DateTimeField(_('agreement date'), blank=True, null=True,
                                          help_text=_('Indicates when the user has agreed with the policy.'))
    preferred_language = models.CharField(max_length=10, blank=True)
    competence = models.CharField(max_length=255, blank=True)
    token_lifetime = models.PositiveIntegerField(null=True, help_text=_('Token lifetime in seconds.'),
                                                 validators=[validators.MinValueValidator(60)])

    tracker = FieldTracker()
    objects = UserManager()

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']

    class Meta(object):
        verbose_name = _('user')
        verbose_name_plural = _('users')

    def get_log_fields(self):
        return ('uuid', 'full_name', 'native_name', self.USERNAME_FIELD, 'is_staff', 'is_support', 'token_lifetime')

    def get_full_name(self):
        # This method is used in django-reversion as name of revision creator.
        return self.full_name

    def get_short_name(self):
        # This method is used in django-reversion as name of revision creator.
        return self.full_name

    def email_user(self, subject, message, from_email=None):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email])

    @classmethod
    def get_permitted_objects_uuids(cls, user):
        if user.is_staff or user.is_support:
            return {'user_uuid': cls.objects.values_list('uuid', flat=True)}
        else:
            return {'user_uuid': [user.uuid]}

    def clean(self):
        super(User, self).clean()
        # User email has to be unique or empty
        if self.email and User.objects.filter(email=self.email).exclude(id=self.id).exists():
            raise ValidationError({'email': _('User with email "%s" already exists.') % self.email})

    def __str__(self):
        if self.full_name:
            return '%s (%s)' % (self.get_username(), self.full_name)

        return self.get_username()


def validate_ssh_public_key(ssh_key):
    # http://stackoverflow.com/a/2494645
    import base64
    import struct

    try:
        key_parts = ssh_key.split(' ', 2)
        key_type, key_body = key_parts[0], key_parts[1]

        if key_type != 'ssh-rsa':
            raise ValidationError(_('Invalid SSH public key type %s, only ssh-rsa is supported.') % key_type)

        data = base64.decodestring(key_body)
        int_len = 4
        # Unpack the first 4 bytes of the decoded key body
        str_len = struct.unpack('>I', data[:int_len])[0]

        encoded_key_type = data[int_len:int_len + str_len]
        # Check if the encoded key type equals to the decoded key type
        if encoded_key_type != key_type:
            raise ValidationError(_("Invalid encoded SSH public key type %s within the key's body, "
                                    "only ssh-rsa is supported.") % encoded_key_type)
    except IndexError:
        raise ValidationError(_('Invalid SSH public key structure.'))

    except (base64.binascii.Error, struct.error):
        raise ValidationError(_('Invalid SSH public key body.'))


def get_ssh_key_fingerprint(ssh_key):
    # How to get fingerprint from ssh key:
    # http://stackoverflow.com/a/6682934/175349
    # http://www.ietf.org/rfc/rfc4716.txt Section 4.
    import base64
    import hashlib

    key_body = base64.b64decode(ssh_key.strip().split()[1].encode('ascii'))
    fp_plain = hashlib.md5(key_body).hexdigest()  # nosec
    return ':'.join(a + b for a, b in zip(fp_plain[::2], fp_plain[1::2]))


@python_2_unicode_compatible
class SshPublicKey(LoggableMixin, UuidMixin, models.Model):
    """
    User public key.

    Used for injection into VMs for remote access.
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL, db_index=True)
    # Model doesn't inherit NameMixin, because name field can be blank.
    name = models.CharField(max_length=150, blank=True)
    fingerprint = models.CharField(max_length=47)  # In ideal world should be unique
    public_key = models.TextField(
        validators=[validators.MaxLengthValidator(2000), validate_ssh_public_key]
    )
    is_shared = models.BooleanField(default=False)

    class Meta(object):
        unique_together = ('user', 'name')
        verbose_name = _('SSH public key')
        verbose_name_plural = _('SSH public keys')

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        # Fingerprint is always set based on public_key
        try:
            self.fingerprint = get_ssh_key_fingerprint(self.public_key)
        except (IndexError, TypeError):
            logger.exception('Fingerprint calculation has failed')
            raise ValueError(_('Public key format is incorrect. Fingerprint calculation has failed.'))

        if update_fields and 'public_key' in update_fields and 'fingerprint' not in update_fields:
            update_fields.append('fingerprint')

        super(SshPublicKey, self).save(force_insert, force_update, using, update_fields)

    def __str__(self):
        return '%s - %s, user: %s, %s' % (self.name, self.fingerprint, self.user.username, self.user.full_name)


class RuntimeStateMixin(models.Model):
    """ Provide runtime_state field """
    class RuntimeStates(object):
        ONLINE = 'online'
        OFFLINE = 'offline'

    class Meta(object):
        abstract = True

    runtime_state = models.CharField(_('runtime state'), max_length=150, blank=True)

    @classmethod
    def get_online_state(cls):
        return cls.RuntimeStates.ONLINE

    @classmethod
    def get_offline_state(cls):
        return cls.RuntimeStates.OFFLINE


class StateMixin(ErrorMessageMixin):
    class States(object):
        CREATION_SCHEDULED = 5
        CREATING = 6
        UPDATE_SCHEDULED = 1
        UPDATING = 2
        DELETION_SCHEDULED = 7
        DELETING = 8
        OK = 3
        ERRED = 4

        CHOICES = (
            (CREATION_SCHEDULED, 'Creation Scheduled'),
            (CREATING, 'Creating'),
            (UPDATE_SCHEDULED, 'Update Scheduled'),
            (UPDATING, 'Updating'),
            (DELETION_SCHEDULED, 'Deletion Scheduled'),
            (DELETING, 'Deleting'),
            (OK, 'OK'),
            (ERRED, 'Erred'),
        )

    class Meta(object):
        abstract = True

    state = FSMIntegerField(
        default=States.CREATION_SCHEDULED,
        choices=States.CHOICES,
    )

    @property
    def human_readable_state(self):
        return force_text(dict(self.States.CHOICES)[self.state])

    @transition(field=state, source=States.CREATION_SCHEDULED, target=States.CREATING)
    def begin_creating(self):
        pass

    @transition(field=state, source=States.UPDATE_SCHEDULED, target=States.UPDATING)
    def begin_updating(self):
        pass

    @transition(field=state, source=States.DELETION_SCHEDULED, target=States.DELETING)
    def begin_deleting(self):
        pass

    @transition(field=state, source=[States.OK, States.ERRED], target=States.UPDATE_SCHEDULED)
    def schedule_updating(self):
        pass

    @transition(field=state, source=[States.OK, States.ERRED], target=States.DELETION_SCHEDULED)
    def schedule_deleting(self):
        pass

    @transition(field=state, source=[States.UPDATING, States.CREATING],
                target=States.OK)
    def set_ok(self):
        pass

    @transition(field=state, source='*', target=States.ERRED)
    def set_erred(self):
        pass

    @transition(field=state, source=States.ERRED, target=States.OK)
    def recover(self):
        pass

    @classmethod
    @lru_cache(maxsize=1)
    def get_all_models(cls):
        return [model for model in apps.get_models() if issubclass(model, cls)]


class ReversionMixin(object):
    """ Store historical values of instance, using django-reversion.

        Note: `ReversionMixin` model should be registered in django-reversion,
              using one of supported methods:
              http://django-reversion.readthedocs.org/en/latest/api.html#registering-models-with-django-reversion
    """

    def get_version_fields(self):
        """ Get field that are tracked in object history versions. """
        options = reversion._get_options(self)
        return options.fields or [f.name for f in self._meta.fields if f not in options.exclude]

    def _is_version_duplicate(self):
        """ Define should new version be created for object or no.

            Reasons to provide custom check instead of default `ignore_revision_duplicates`:
             - no need to compare all revisions - it is OK if right object version exists in any revision;
             - need to compare object attributes (not serialized data) to avoid
               version creation on wrong <float> vs <int> comparison;
        """
        if self.id is None:
            return False
        try:
            latest_version = Version.objects.get_for_object(self).latest('revision__date_created')
        except Version.DoesNotExist:
            return False
        latest_version_object = latest_version._object_version.object
        fields = self.get_version_fields()
        return all([getattr(self, f) == getattr(latest_version_object, f) for f in fields])

    def save(self, **kwargs):
        if self._is_version_duplicate():
            return super(ReversionMixin, self).save(**kwargs)
        with reversion.create_revision():
            return super(ReversionMixin, self).save(**kwargs)


# XXX: consider renaming it to AffinityMixin
class DescendantMixin(object):
    """ Mixin to provide child-parent relationships.
        Each related model can provide list of its parents/children.
    """
    def get_parents(self):
        """ Return list instance parents. """
        return []

    def get_children(self):
        """ Return list instance children. """
        return []

    def get_ancestors(self):
        """ Get all unique instance ancestors """
        ancestors = list(self.get_parents())
        ancestor_unique_attributes = set([(a.__class__, a.id) for a in ancestors])
        ancestors_with_parents = [a for a in ancestors if isinstance(a, DescendantMixin)]
        for ancestor in ancestors_with_parents:
            for parent in ancestor.get_ancestors():
                if (parent.__class__, parent.id) not in ancestor_unique_attributes:
                    ancestors.append(parent)
        return ancestors

    def get_descendants(self):
        def traverse(obj):
            for child in obj.get_children():
                yield child
                for baby in child.get_descendants():
                    yield baby
        return list(set(traverse(self)))


class AbstractFieldTracker(FieldTracker):
    """
    Workaround for abstract models
    https://gist.github.com/sbnoemi/7618916
    """
    def finalize_class(self, sender, name, **kwargs):
        self.name = name
        self.attname = '_%s' % name
        if not hasattr(sender, name):
            super(AbstractFieldTracker, self).finalize_class(sender, **kwargs)


class BackendModelMixin(object):
    """
    Represents model that is connected to backend object. 

    This model cannot be created or updated via admin, because we do not support queries to backend from admin interface.
    """

    @classmethod
    def get_backend_fields(cls):
        """
        Returns a list of fields that are handled on backend.
        """
        return ()



from django.apps import apps

from waldur_core.logging.loggers import LoggableMixin


def get_loggable_models():
    return [model for model in apps.get_models() if issubclass(model, LoggableMixin)]


def get_scope_types_mapping():
    return {str(m._meta): m for m in get_loggable_models()}


def get_reverse_scope_types_mapping():
    return {m: str(m._meta) for m in get_loggable_models()}



from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import redirect
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import ungettext, ugettext_lazy as _

from waldur_core.core import admin as core_admin, utils as core_utils
from waldur_core.cost_tracking import models, CostTrackingRegister, ResourceNotRegisteredError, tasks
from waldur_core.structure import SupportedServices
from waldur_core.structure import models as structure_models, admin as structure_admin


def _get_content_type_queryset(models_list):
    """ Get list of services content types """
    content_type_ids = {c.id for c in ContentType.objects.get_for_models(*models_list).values()}
    return ContentType.objects.filter(id__in=content_type_ids)


class ResourceTypeFilter(SimpleListFilter):
    title = 'resource_type'
    parameter_name = 'resource_type'

    def lookups(self, request, model_admin):
        return [(name, name) for name, model in SupportedServices.get_resource_models().items()
                if model in CostTrackingRegister.registered_resources]

    def queryset(self, request, queryset):
        if self.value():
            model = SupportedServices.get_resource_models().get(self.value(), None)
            if model:
                return queryset.filter(resource_content_type=ContentType.objects.get_for_model(model))
        return queryset


class DefaultPriceListItemAdmin(core_admin.ExtraActionsMixin, structure_admin.ChangeReadonlyMixin, admin.ModelAdmin):
    list_display = ('full_name', 'item_type', 'key', 'value', 'monthly_rate', 'resource_type')
    list_filter = ('item_type', ResourceTypeFilter)
    fields = ('name', ('value', 'monthly_rate'), 'resource_content_type', ('item_type', 'key'))
    readonly_fields = ('monthly_rate',)
    change_readonly_fields = ('resource_content_type', 'item_type', 'key')

    def full_name(self, obj):
        return obj.name or obj.units or obj.uuid

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "resource_content_type":
            kwargs["queryset"] = _get_content_type_queryset(CostTrackingRegister.registered_resources.keys())
        return super(DefaultPriceListItemAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def get_extra_actions(self):
        return [
            self.init_registered,
            self.delete_not_registered,
            self.recalulate_current_estimates,
            self.reinit_configurations,
        ]

    def init_registered(self, request):
        """ Create default price list items for each registered resource. """
        created_items = models.DefaultPriceListItem.init_from_registered_resources()

        if created_items:
            message = ungettext(
                _('Price item was created: %s.') % created_items[0].name,
                _('Price items were created: %s.') % ', '.join(item.name for item in created_items),
                len(created_items)
            )
            self.message_user(request, message)
        else:
            self.message_user(request, _('Price items for all registered resources have been updated.'))

        return redirect(reverse('admin:cost_tracking_defaultpricelistitem_changelist'))

    def delete_not_registered(self, request):
        deleted_items_names = []

        for price_list_item in models.DefaultPriceListItem.objects.all():
            try:
                resource_class = price_list_item.resource_content_type.model_class()
                consumable_items = CostTrackingRegister.get_consumable_items(resource_class)
                next(item for item in consumable_items
                     if item.key == price_list_item.key and item.item_type == price_list_item.item_type)
            except (ResourceNotRegisteredError, StopIteration):
                deleted_items_names.append(price_list_item.name)
                price_list_item.delete()

        if deleted_items_names:
            message = ungettext(
                _('Price item was deleted: %s.') % deleted_items_names[0],
                _('Price items were deleted: %s.') % ', '.join(item for item in deleted_items_names),
                len(deleted_items_names)
            )
            self.message_user(request, message)
        else:
            self.message_user(request, _('Nothing to delete. All default price items are registered.'))

        return redirect(reverse('admin:cost_tracking_defaultpricelistitem_changelist'))

    def recalulate_current_estimates(self, request):
        tasks.recalculate_estimate(recalculate_total=True)
        self.message_user(request, _('Total and consumed value were successfully recalculated for all price estimates.'))
        return redirect(reverse('admin:cost_tracking_defaultpricelistitem_changelist'))

    def reinit_configurations(self, request):
        """ Re-initialize configuration for resource if it has been changed.

            This method should be called if resource consumption strategy was changed.
        """
        now = timezone.now()

        # Step 1. Collect all resources with changed configuration.
        changed_resources = []
        for resource_model in CostTrackingRegister.registered_resources:
            for resource in resource_model.objects.all():
                try:
                    pe = models.PriceEstimate.objects.get(scope=resource, month=now.month, year=now.year)
                except models.PriceEstimate.DoesNotExist:
                    changed_resources.append(resource)
                else:
                    new_configuration = CostTrackingRegister.get_configuration(resource)
                    if new_configuration != pe.consumption_details.configuration:
                        changed_resources.append(resource)

        # Step 2. Re-init configuration and recalculate estimate for changed resources.
        for resource in changed_resources:
            models.PriceEstimate.update_resource_estimate(resource, CostTrackingRegister.get_configuration(resource))

        message = _('Configuration was reinitialized for %(count)s resources') % {'count': len(changed_resources)}
        self.message_user(request, message)

        return redirect(reverse('admin:cost_tracking_defaultpricelistitem_changelist'))


class PriceListItemAdmin(admin.ModelAdmin):
    list_display = ('uuid', 'default_price_list_item', 'service', 'units', 'value')

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "content_type":
            kwargs["queryset"] = _get_content_type_queryset(structure_models.Service.get_all_models())
        return super(PriceListItemAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)


class ScopeTypeFilter(SimpleListFilter):
    title = 'resource_type'
    parameter_name = 'resource_type'

    def lookups(self, request, model_admin):
        resources = [(model, name) for name, model in SupportedServices.get_resource_models().items()]
        others = [(model, model.__name__) for model in models.PriceEstimate.get_estimated_models()
                  if not issubclass(model, structure_models.ResourceMixin)]
        estimated_models = [(core_utils.serialize_class(model), name) for model, name in resources + others]
        return sorted(estimated_models, key=lambda x: x[1])

    def queryset(self, request, queryset):
        if self.value():
            model = core_utils.deserialize_class(self.value())
            return queryset.filter(content_type=ContentType.objects.get_for_model(model))
        return queryset


class ConsumptionDetailsInline(admin.StackedInline):
    model = models.ConsumptionDetails
    readonly_fields = ('configuration', 'last_update_time', 'consumed_before_update')
    extra = 0
    can_delete = False


class PriceEstimateAdmin(admin.ModelAdmin):
    inlines = [ConsumptionDetailsInline]
    fields = ('content_type', 'object_id', ('total', 'consumed'), ('month', 'year'))
    list_display = ('content_type', 'object_id', 'total', 'month', 'year')
    list_filter = (ScopeTypeFilter, 'year', 'month')
    search_fields = ('month', 'year', 'object_id', 'total')


admin.site.register(models.DefaultPriceListItem, DefaultPriceListItemAdmin)
admin.site.register(models.PriceListItem, PriceListItemAdmin)
admin.site.register(models.PriceEstimate, PriceEstimateAdmin)



from django.test import TestCase

from waldur_core.structure.tests import factories


class ServiceProjectLinkTest(TestCase):

    def setUp(self):
        self.link = factories.TestServiceProjectLinkFactory()

    def test_link_validation_state_is_ERRED_if_service_does_not_satisfy_project_certifications(self):
        certification = factories.ServiceCertificationFactory()
        self.assertEqual(self.link.States.OK, self.link.validation_state)

        self.link.project.certifications.add(certification)

        self.assertEqual(self.link.States.ERRED, self.link.validation_state)

    def test_link_validation_state_is_OK_if_project_certifications_is_a_subset_of_service_certifications(self):
        certifications = factories.ServiceCertificationFactory.create_batch(2)
        self.link.project.certifications.add(*certifications)
        certifications.append(factories.ServiceCertificationFactory())

        self.link.service.settings.certifications.add(*certifications)

        self.assertEqual(self.link.States.OK, self.link.validation_state)




from admin_tools.menu import items, Menu
from django.urls import reverse
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _


class CustomAppList(items.AppList):
    def init_with_context(self, context):
        context_items = self._visible_models(context['request'])
        apps = {}
        for model, perms in context_items:
            if not perms['change']:
                continue
            app_label = model._meta.app_label
            if app_label not in apps:
                apps[app_label] = {
                    'title': capfirst(model._meta.app_config.verbose_name),
                    'url': self._get_admin_app_list_url(model, context),
                    'models': []
                }
            apps[app_label]['models'].append({
                'title': capfirst(model._meta.verbose_name_plural),
                'url': self._get_admin_change_url(model, context)
            })

        for app in sorted(apps, key=lambda k: apps[k]['title']):
            app_dict = apps[app]
            item = items.MenuItem(title=app_dict['title'], url=app_dict['url'])
            # sort model list alphabetically
            apps[app]['models'].sort(key=lambda x: x['title'])
            for model_dict in apps[app]['models']:
                item.children.append(items.MenuItem(**model_dict))
            self.children.append(item)


class CustomMenu(Menu):
    """
    Custom Menu for admin site.
    """

    IAAS_CLOUDS = (
        'waldur_mastermind.packages.*',
        'waldur_azure.*',
        'waldur_openstack.*',
        'waldur_aws.*',
        'waldur_digitalocean.*',
        'waldur_slurm.*',
        'waldur_mastermind.slurm_invoices.*',
    )

    USERS = (
        'waldur_core.core.models.*',
    )

    ACCOUNTING = (
        'waldur_mastermind.invoices.*',
        'waldur_core.cost_tracking.*',
    )

    APPLICATION_PROVIDERS = (
        'waldur_ansible.*',
    )

    SUPPORT_MODULES = (
        'waldur_mastermind.support.*',
    )

    def __init__(self, **kwargs):
        Menu.__init__(self, **kwargs)
        self.children += [
            items.MenuItem(_('Dashboard'), reverse('admin:index')),
            items.ModelList(
                _('Users'),
                models=self.USERS
            ),
            items.ModelList(
                _('Structure'),
                models=(
                    'waldur_core.structure.*',
                )
            ),
            CustomAppList(
                _('Accounting'),
                models=self.ACCOUNTING,
            ),

            CustomAppList(
                _('Providers'),
                models=self.IAAS_CLOUDS,
            ),
            CustomAppList(
                _('Applications'),
                models=self.APPLICATION_PROVIDERS,
            ),
            CustomAppList(
                _('Support'),
                models=self.SUPPORT_MODULES,
            ),
            CustomAppList(
                _('Utilities'),
                exclude=('django.core.*',
                         'django_openid_auth.*',
                         'rest_framework.authtoken.*',
                         'waldur_core.core.*',
                         'waldur_core.structure.*',
                         )
                        + self.IAAS_CLOUDS
                        + self.APPLICATION_PROVIDERS
                        + self.SUPPORT_MODULES
                        + self.ACCOUNTING
                        + self.USERS
            ),

        ]



""" Custom loggers that allows to store logs in DB and Elastic """

import uuid
import types
import decimal
import datetime
import importlib
import logging
from collections import defaultdict

from django.apps import apps
from django.contrib.contenttypes import models as ct_models
from django.db import transaction, IntegrityError
from django.utils import six

from waldur_core.logging import models
from waldur_core.logging.log import EventLoggerAdapter
from waldur_core.logging.middleware import get_event_context


logger = logging.getLogger(__name__)


class LoggerError(AttributeError):
    pass


class EventLoggerError(AttributeError):
    pass


class AlertLoggerError(AttributeError):
    pass


class BaseLogger(object):
    def __init__(self, logger_name=__name__):
        self._meta = getattr(self, 'Meta', None)
        self.supported_types = self.get_supported_types()

    def get_supported_types(self):
        raise NotImplementedError

    def get_supported_groups(self):
        raise NotImplementedError

    def get_nullable_fields(self):
        return getattr(self._meta, 'nullable_fields', [])

    def get_field_model(self, model):
        if not isinstance(model, basestring):
            return model

        try:
            return apps.get_model(model)
        except LookupError:
            try:
                app_name, class_name = model.split('.')
                module = importlib.import_module('waldur_core.%s.models' % app_name)
                return getattr(module, class_name)
            except (ImportError, AttributeError, IndexError):
                raise LoggerError("Can't find model %s" % model)

    def compile_message(self, message_template, context):
        try:
            msg = six.text_type(message_template).format(**context)
        except KeyError as e:
            raise LoggerError(
                "Cannot find %s context field. Choices are: %s" % (
                    str(e), ', '.join(context.keys())))
        return msg

    def validate_logging_type(self, logging_type):
        if self.supported_types and logging_type not in self.supported_types:
            raise EventLoggerError(
                "Unsupported logging type '%s'. Choices are: %s" % (
                    logging_type, ', '.join(self.supported_types)))

    def compile_context(self, **kwargs):
        # Get a list of fields here in order to be sure all models already loaded.
        if not hasattr(self, 'fields'):
            self.fields = {
                k: self.get_field_model(v)
                for k, v in self.__class__.__dict__.items()
                if not k.startswith('_') and not isinstance(v, (types.ClassType, types.FunctionType))}

        missed = set(self.fields.keys()) - set(self.get_nullable_fields()) - set(kwargs.keys())
        if missed:
            raise LoggerError("Missed fields in event context: %s" % ', '.join(missed))

        context = {}

        event_context = get_event_context()
        if event_context:
            context.update(event_context)
            username = event_context.get('user_username')
            if 'user' in self.fields and username:
                logger.warning("User is passed directly to event context. "
                               "Currently authenticated user %s is ignored.", username)

        for entity_name, entity in six.iteritems(kwargs):
            if entity_name in self.fields:
                entity_class = self.fields[entity_name]
                if entity is None and entity_name in self.get_nullable_fields():
                    continue
                if not isinstance(entity, entity_class):
                    raise LoggerError(
                        "Field '%s' must be an instance of %s but %s received" % (
                            entity_name, entity_class.__name__, entity.__class__.__name__))
            else:
                logger.error(
                    "Field '%s' cannot be used in logging context for %s",
                    entity_name, self.__class__.__name__)
                continue

            if isinstance(entity, LoggableMixin):
                context.update(entity._get_log_context(entity_name))
            elif isinstance(entity, (int, float, basestring, dict, tuple, list, bool)):
                context[entity_name] = entity
            elif entity is None:
                pass
            else:
                context[entity_name] = six.text_type(entity)
                logger.warning(
                    "Cannot properly serialize '%s' context field. "
                    "Must be inherited from LoggableMixin." % entity_name)

        return context


class EventLogger(BaseLogger):
    """ Base event logger API.
        Fields which must be passed during event log emitting (event context)
        should be defined as attributes for this class in the form of:

        field_name = ObjectClass || '<app_label>.<class_name>'

        A list of supported event types can be defined with help of method get_supported_types,
        or 'event_types' property of Meta class. Event type won't be validated if this list is empty.

        Example usage:

        .. code-block:: python

            from waldur_core.openstack.models import Tenant

            class QuotaEventLogger(EventLogger):
                tenant = Tenant
                project = 'structure.Project'
                threshold = float
                quota_type = basestring

                class Meta:
                    event_types = 'quota_threshold_reached',


            quota_logger = QuotaEventLogger(__name__)
            quota_logger.warning(
                '{quota_type} quota threshold has been reached for {project_name}.',
                event_type='quota_threshold_reached',
                event_context=dict(
                    quota_type=quota.name,
                    project=membership.project,
                    tenant=tenant,
                    threshold=threshold * quota.limit)
            )
    """

    def __init__(self, logger_name=__name__):
        super(EventLogger, self).__init__(logger_name)
        self.logger = EventLoggerAdapter(logging.getLogger(logger_name))

    def get_supported_types(self):
        return getattr(self._meta, 'event_types', tuple())

    def get_supported_groups(self):
        return getattr(self._meta, 'event_groups', {})

    def info(self, *args, **kwargs):
        self.process('info', *args, **kwargs)

    def error(self, *args, **kwargs):
        self.process('error', *args, **kwargs)

    def warning(self, *args, **kwargs):
        self.process('warning', *args, **kwargs)

    def debug(self, *args, **kwargs):
        self.process('debug', *args, **kwargs)

    def process(self, level, message_template, event_type='undefined', event_context=None):
        self.validate_logging_type(event_type)

        if not event_context:
            event_context = {}

        context = self.compile_context(**event_context)
        msg = self.compile_message(message_template, context)

        log = getattr(self.logger, level)
        log(msg, extra={'event_type': event_type, 'event_context': context})


class AlertLogger(BaseLogger):
    """ Base alert logger API.

        Fields which must be passed during alert log emitting (alert context)
        should be defined as attributes for this class in the form of:

        field_name = ObjectClass || '<app_label>.<class_name>'

        A list of supported event types can be defined with help of method get_supported_types,
        or 'alert_types' property of Meta class. Event type won't be validated if this list is empty.

        Example usage:

        .. code-block:: python

            from waldur_core.logging.loggers import AlertLogger, alert_logger
            from waldur_core.quotas import models

            class QuotaAlertLogger(AlertLogger):
                quota = models.Quota

                class Meta:
                    alert_types = ('quota_usage_is_over_threshold', )

            alert_logger.register('quota', QuotaAlertLogger)


            alert_logger.quota.warning(
                'Quota {quota_name} is over threshold. Limit: {quota_limit}, usage: {quota_usage}',
                scope=quota,
                alert_type='quota_usage_is_over_threshold',
                alert_context={
                    'quota': quota
                })
    """

    def get_supported_types(self):
        return getattr(self._meta, 'alert_types', tuple())

    def get_supported_groups(self):
        return getattr(self._meta, 'alert_groups', {})

    def info(self, *args, **kwargs):
        return self.process(models.Alert.SeverityChoices.INFO, *args, **kwargs)

    def error(self, *args, **kwargs):
        return self.process(models.Alert.SeverityChoices.ERROR, *args, **kwargs)

    def warning(self, *args, **kwargs):
        return self.process(models.Alert.SeverityChoices.WARNING, *args, **kwargs)

    def debug(self, *args, **kwargs):
        return self.process(models.Alert.SeverityChoices.DEBUG, *args, **kwargs)

    def process(self, severity, message_template, scope, alert_type='undefined', alert_context=None, fail_silently=True):
        self.validate_logging_type(alert_type)

        if not alert_context:
            alert_context = {}

        context = self.compile_context(**alert_context)
        msg = self.compile_message(message_template, context)
        content_type = ct_models.ContentType.objects.get_for_model(scope)

        try:
            with transaction.atomic():
                alert = models.Alert.objects.select_for_update().get(
                    content_type=content_type,
                    object_id=scope.id,
                    alert_type=alert_type,
                    closed__isnull=True
                )
                if alert.severity != severity or alert.message != msg:
                    alert.severity = severity
                    alert.message = msg
                    alert.save()

                    logger.info(
                        'Updated alert for scope %s (id: %s), with type %s',
                        scope, scope.id, alert_type)

                return alert, False
        except models.Alert.DoesNotExist:
            pass

        try:
            alert = models.Alert.objects.create(
                scope=scope,
                alert_type=alert_type,
                severity=severity,
                message=msg,
                context=context
            )
            logger.info(
                'Created new alert for scope %s (id: %s), with type %s',
                scope, scope.id, alert_type)
            return alert, True
        except IntegrityError:
            logger.warning(
                'Could not create alert for scope %s (id: %s), with type %s due to concurrent update',
                scope, scope.id, alert_type)
            if fail_silently:
                return None, False
            else:
                raise

    def close(self, scope, alert_type):
        try:
            content_type = ct_models.ContentType.objects.get_for_model(scope)
            alert = models.Alert.objects.get(
                object_id=scope.id, content_type=content_type, alert_type=alert_type, closed__isnull=True)
            alert.close()
        except models.Alert.DoesNotExist:
            pass


class LoggableMixin(object):
    """ Mixin to serialize model in logs.
        Extends django model or custom class with fields extraction method.
    """

    def get_log_fields(self):
        return ('uuid', 'name')

    def filter_by_logged_object(self):
        """
        Return query dictionary to search current object in ElasticSearch.
        Model may return custom query, but backend doesn't need to know details.
        """
        return {
            self.__class__.__name__ + '_uuid': self.uuid.hex
        }

    def _get_log_context(self, entity_name):

        context = {}
        for field in self.get_log_fields():
            if not hasattr(self, field):
                continue

            value = getattr(self, field)

            name = "{}_{}".format(entity_name, field)
            if isinstance(value, uuid.UUID):
                context[name] = value.hex
            elif isinstance(value, LoggableMixin):
                context.update(value._get_log_context(field))
            elif isinstance(value, datetime.date):
                context[name] = value.isoformat()
            elif isinstance(value, decimal.Decimal):
                context[name] = float(value)
            elif isinstance(value, dict):
                context[name] = value
            else:
                context[name] = six.text_type(value)

        return context

    @classmethod
    def get_permitted_objects_uuids(cls, user):
        return {}


class BaseLoggerRegistry(object):

    def get_loggers(self):
        raise NotImplementedError('Method "get_loggers" is not implemented.')

    def register(self, name, logger):
        if name in self.__dict__:
            raise EventLoggerError("Logger '%s' already registered." % name)
        self.__dict__[name] = logger() if isinstance(logger, type) else logger

    def unregister_all(self):
        self.__dict__ = {}

    def get_all_types(self):
        events = set()
        for elogger in self.get_loggers():
            events.update(elogger.get_supported_types())
        return list(sorted(events))

    def get_all_groups(self):
        groups = defaultdict(set)
        for elogger in self.get_loggers():
            for group, items in elogger.get_supported_groups().items():
                groups[group].update(set(items))
        return dict(groups)

    def expand_groups(self, groups):
        items = set()
        mapping = self.get_all_groups()
        for group in groups:
            items.update(mapping.get(group, []))
        return sorted(items)


class EventLoggerRegistry(BaseLoggerRegistry):

    def get_loggers(self):
        return [l for l in self.__dict__.values() if isinstance(l, EventLogger)]

    def get_permitted_objects_uuids(self, user):
        from waldur_core.logging.utils import get_loggable_models
        permitted_objects_uuids = {}
        for model in get_loggable_models():
            for field, uuids in model.get_permitted_objects_uuids(user).items():
                permitted_objects_uuids[field] = [uuid.hex for uuid in uuids]
        return permitted_objects_uuids


class AlertLoggerRegistry(BaseLoggerRegistry):

    def get_loggers(self):
        return [l for l in self.__dict__.values() if isinstance(l, AlertLogger)]


def get_valid_events():
    return event_logger.get_all_types()


def get_valid_alerts():
    return alert_logger.get_all_types()


def get_event_groups():
    return event_logger.get_all_groups()


def get_event_groups_keys():
    return sorted(get_event_groups().keys())


def get_alert_groups():
    return alert_logger.get_all_groups()


def expand_event_groups(groups):
    return event_logger.expand_groups(groups)


def expand_alert_groups(groups):
    return alert_logger.expand_groups(groups)


class CustomEventLogger(EventLogger):
    """
    Logger for custom events e.g. created by staff user
    """
    scope = LoggableMixin

    class Meta:
        event_types = ('custom_notification',)
        nullable_fields = ('scope',)


class ThresholdAlertLogger(AlertLogger):
    object = 'logging.AlertThresholdMixin'

    class Meta:
        alert_types = ('threshold_exceeded',)


# This global objects represent the default loggers registry
event_logger = EventLoggerRegistry()
alert_logger = AlertLoggerRegistry()

event_logger.register('custom', CustomEventLogger)
alert_logger.register('threshold', ThresholdAlertLogger)



from django.contrib.contenttypes import models as ct_models
from django.db import models
from django.db.models import Q

from waldur_core.core.managers import GenericKeyMixin


class QuotaManager(GenericKeyMixin, models.Manager):

    def filtered_for_user(self, user, queryset=None):
        from waldur_core.quotas import utils

        if queryset is None:
            queryset = self.get_queryset()
        # XXX: This circular dependency will be removed then filter_queryset_for_user
        # will be moved to model manager method
        from waldur_core.structure.managers import filter_queryset_for_user

        if user.is_staff or user.is_support:
            return queryset

        quota_scope_models = utils.get_models_with_quotas()
        query = Q()
        for model in quota_scope_models:
            user_object_ids = filter_queryset_for_user(model.objects.all(), user).values_list('id', flat=True)
            content_type_id = ct_models.ContentType.objects.get_for_model(model).id
            query |= Q(object_id__in=user_object_ids, content_type_id=content_type_id)

        return queryset.filter(query)



# This file mainly exists to allow python setup.py test to work.
import os
import sys

from setuptools.command.test import test as TestCommand

os.environ['DJANGO_SETTINGS_MODULE'] = 'waldur_core.server.test_settings'
test_dir = os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), os.path.pardir))
sys.path.insert(0, test_dir)


class Test(TestCommand):
    user_options = TestCommand.user_options + [('parallel', None, "Runs tests in separate parallel processes.")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.parallel = 0

    def run_tests(self):
        import django
        from django.conf import settings
        from django.test.runner import default_test_processes
        from django.test.utils import get_runner

        django.setup()
        test_runner_class = get_runner(settings)
        try:
            import xmlrunner

            class XMLTestRunner(test_runner_class):
                def run_suite(self, suite, **kwargs):
                    verbosity = getattr(settings, 'TEST_OUTPUT_VERBOSE', 1)
                    if isinstance(verbosity, bool):
                        verbosity = (1, 2)[verbosity]
                    descriptions = getattr(settings, 'TEST_OUTPUT_DESCRIPTIONS', False)
                    output = getattr(settings, 'TEST_OUTPUT_DIR', '.')

                    return xmlrunner.XMLTestRunner(
                        verbosity=verbosity,
                        descriptions=descriptions,
                        output=output
                    ).run(suite)
            test_runner_class = XMLTestRunner
        except ImportError:
            print "Not generating XML reports, run 'pip install unittest-xml-reporting' to enable XML report generation"

        if self.parallel:
            parallel = default_test_processes()
        else:
            parallel = 0

        test_runner = test_runner_class(verbosity=2, interactive=True, parallel=parallel)

        failures = test_runner.run_tests([])
        sys.exit(bool(failures))



import os
import tempfile

from django.db import models


def get_upload_path(instance, filename):
    path = '%s/%s' % (instance._meta.model_name, instance.uuid.hex)
    _, ext = os.path.splitext(filename)
    return '%s%s' % (path, ext)


def dummy_image(filetype='gif'):
    """ Generate empty image in temporary file for testing """
    # 1x1px Transparent GIF
    GIF = 'R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7'
    tmp_file = tempfile.NamedTemporaryFile(suffix='.%s' % filetype)
    tmp_file.write(GIF.decode('base64'))
    return open(tmp_file.name)


class ImageModelMixin(models.Model):
    class Meta(object):
        abstract = True

    image = models.ImageField(upload_to=get_upload_path, null=True, blank=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('logging', '0007_pushhook_unique_key'),
    ]

    operations = [
        migrations.AddField(
            model_name='pushhook',
            name='token',
            field=models.CharField(max_length=255, unique=True, null=True),
        ),
    ]



"""
Cost tracking - add-on for NC plugins.
Allows to calculate price estimates for resources from your plugin.

Check developer guide for more details.
"""
import logging

from django.utils.module_loading import autodiscover_modules


default_app_config = 'waldur_core.cost_tracking.apps.CostTrackingConfig'
logger = logging.getLogger(__name__)


class ConsumableItem(object):

    def __init__(self, item_type, key, name=None, units='', default_price=0):
        self.item_type = item_type
        self.key = key
        self.default_price = default_price
        self.name = name if name is not None else '%s: %s' % (item_type, key)
        self.units = units

    def __repr__(self):
        return 'ConsumableItem(%s)' % self.name

    def __str__(self):
        return self.name

    def __hash__(self):
        return hash((self.item_type, self.key))

    def __eq__(self, other):
        return (self.item_type, self.key) == (other.item_type, other.key)

    def __ne__(self, other):
        # Not strictly necessary, but to avoid having both x==y and x!=y True at the same time
        return not(self == other)


class CostTrackingStrategy(object):
    """ Describes all methods that should be implemented to enable cost
        tracking for particular resource.
    """
    resource_class = NotImplemented

    @classmethod
    def get_configuration(cls, resource):
        """ Return dictionary of consumables that are used by resource.

            Dictionary key - ConsumableItem instance.
            Dictionary value - how many units of consumable is used by resource.
            Example: {
                ConsumableItem('storage', '1 MB'): 10240,
                ConsumableItem('flavor', 'small'): 1,
                ...
            }
        """
        return {}

    @classmethod
    def get_consumable_items(cls):
        """ Return list of all possible consumed items.

            Output format:
            [
                ConsumableItem(
                    item_type=<type of consumable>,
                    key=<consumable name>,
                    name=<item pretty name, that will be visible for user>,
                    units=<consumable units (MB, GB, points, etc.>,
                    default_price=<price for consumable usage per hour>,
                )
                ...
            ]
            Output example:
            [
                ConsumableItem(
                    item_type="storage"
                    key="1 MB",
                    units="MB",
                    name="1 MB of storage",
                    default_price=0.5,
                ),
                ConsumableItem(
                    item_type="flavor"
                    key="small",
                    name="Small flavor",
                ),
                ...
            ]
        """
        return []


class ResourceNotRegisteredError(TypeError):
    pass


class CostTrackingRegister(object):
    """ Register of all connected NC plugins """
    registered_resources = {}
    is_autodiscovered = False

    @classmethod
    def autodiscover(cls):
        if not cls.is_autodiscovered:
            autodiscover_modules('cost_tracking')
            cls.is_autodiscovered = True

    @classmethod
    def register_strategy(cls, strategy):
        cls.registered_resources[strategy.resource_class] = strategy

    @classmethod
    def _get_strategy(cls, resource_class):
        try:
            return cls.registered_resources[resource_class]
        except KeyError:
            raise ResourceNotRegisteredError('Resource %s is not registered for cost tracking. Make sure that its '
                                             'strategy is added to CostTrackingRegister' % resource_class.__name__)

    @classmethod
    def get_configuration(cls, resource):
        """ Return how much consumables are used by resource with current configuration.

            Output example:
            {
                <ConsumableItem instance>: <usage>,
                <ConsumableItem instance>: <usage>,
                ...
            }
        """
        strategy = cls._get_strategy(resource.__class__)
        return strategy.get_configuration(resource)

    @classmethod
    def get_consumable_items(cls, resource_class):
        """ Get all possible consumable items for given resource class """
        strategy = cls._get_strategy(resource_class)
        return strategy.get_consumable_items()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    replaces = [('logging', '0002_index_alert_type')]

    dependencies = [
        ('logging', '0001_squashed_0003_emailhook_webhook'),
    ]

    operations = [
        migrations.AlterField(
            model_name='alert',
            name='alert_type',
            field=models.CharField(max_length=50, db_index=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import re
import waldur_core.core.models
import waldur_core.core.fields
import django.utils.timezone
from django.conf import settings
import django.contrib.auth.models
import django.core.validators


class Migration(migrations.Migration):

    replaces = [('core', '0001_initial'), ('core', '0002_user_organization_approved'), ('core', '0003_ssh_key_name_length_changed')]

    dependencies = [
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(null=True, verbose_name='last login', blank=True)),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('description', models.CharField(max_length=500, verbose_name='description', blank=True)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('username', models.CharField(help_text='Required. 30 characters or fewer. Letters, numbers and @/./+/-/_ characters', unique=True, max_length=30, verbose_name='username', validators=[django.core.validators.RegexValidator(re.compile('^[\\w.@+-]+$'), 'Enter a valid username.', 'invalid')])),
                ('civil_number', models.CharField(null=True, default=None, max_length=10, blank=True, unique=True, verbose_name='civil number')),
                ('full_name', models.CharField(max_length=100, verbose_name='full name', blank=True)),
                ('native_name', models.CharField(max_length=100, verbose_name='native name', blank=True)),
                ('phone_number', models.CharField(max_length=40, verbose_name='phone number', blank=True)),
                ('organization', models.CharField(max_length=80, verbose_name='organization', blank=True)),
                ('job_title', models.CharField(max_length=40, verbose_name='job title', blank=True)),
                ('email', models.EmailField(max_length=75, verbose_name='email address', blank=True)),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('groups', models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Group', blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(related_query_name='user', related_name='user_set', to=b'auth.Permission', blank=True, help_text='Specific permissions for this user.', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SshPublicKey',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('name', models.CharField(max_length=150, blank=True)),
                ('fingerprint', models.CharField(max_length=47)),
                ('public_key', models.TextField(validators=[django.core.validators.MaxLengthValidator(2000), waldur_core.core.models.validate_ssh_public_key])),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': set([('user', 'name')]),
                'verbose_name': 'SSH public key',
                'verbose_name_plural': 'SSH public keys'
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='user',
            name='organization_approved',
            field=models.BooleanField(default=False, help_text='Designates whether user organization was approved.', verbose_name='organization approved'),
            preserve_default=True,
        ),
    ]



from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth import get_user_model
from django.utils import timezone


class Command(BaseCommand):
    help = "Create a user with a specified username and password. User will be created as staff."

    def add_arguments(self, parser):
        parser.add_argument('-u', '--username', dest='username', required=True)
        parser.add_argument('-p', '--password', dest='password', required=True)

    def handle(self, *args, **options):
        User = get_user_model()

        username = options['username']
        password = options['password']

        user, created = User.objects.get_or_create(
            username=username, defaults=dict(last_login=timezone.now(), is_staff=True))
        if not created:
            raise CommandError('Username %s is already taken.' % username)

        user.set_password(password)
        user.save()
        self.stdout.write(self.style.SUCCESS('User %s has been created.' % username))



from rest_framework import status, test
from rest_framework.reverse import reverse

from waldur_core.structure.tests import factories


class ServiceMetadataTest(test.APITransactionTestCase):
    def test_any_user_can_get_service_metadata(self):
        self.client.force_authenticate(factories.UserFactory())
        response = self.client.get(reverse('service_metadata-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0038_add_project_and_customer_permissions'),
        ('users', '0004_migrate_to_new_permissions_model'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='customerrole',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='customerrole',
            name='customer',
        ),
        migrations.RemoveField(
            model_name='customerrole',
            name='permission_group',
        ),
        migrations.RemoveField(
            model_name='projectgroup',
            name='customer',
        ),
        migrations.RemoveField(
            model_name='projectgroup',
            name='projects',
        ),
        migrations.AlterUniqueTogether(
            name='projectgrouprole',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='projectgrouprole',
            name='permission_group',
        ),
        migrations.RemoveField(
            model_name='projectgrouprole',
            name='project_group',
        ),
        migrations.AlterUniqueTogether(
            name='projectrole',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='projectrole',
            name='permission_group',
        ),
        migrations.RemoveField(
            model_name='projectrole',
            name='project',
        ),
        migrations.DeleteModel(
            name='CustomerRole',
        ),
        migrations.DeleteModel(
            name='ProjectGroup',
        ),
        migrations.DeleteModel(
            name='ProjectGroupRole',
        ),
        migrations.DeleteModel(
            name='ProjectRole',
        ),
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth import get_user_model
from django.test import TestCase
from django.utils import timezone


class TestModels(TestCase):

    def test_token_lifetime_is_read_from_settings_as_default_value_when_user_is_created(self):
        waldur_core_settings = settings.WALDUR_CORE.copy()
        waldur_core_settings['TOKEN_LIFETIME'] = timezone.timedelta(days=1)

        with self.settings(WALDUR_CORE=waldur_core_settings):
            token_lifetime = settings.WALDUR_CORE['TOKEN_LIFETIME']
            expected_lifetime = int(token_lifetime.total_seconds())
            user = get_user_model().objects.create(username='test1')
            self.assertEqual(user.token_lifetime, expected_lifetime)



from __future__ import unicode_literals

from django.apps import AppConfig
from django.db.models import signals
from django_fsm import signals as fsm_signals


class StructureConfig(AppConfig):
    name = 'waldur_core.structure'
    verbose_name = 'Structure'

    def ready(self):
        from waldur_core.core.models import CoordinatesMixin
        from waldur_core.structure.executors import check_cleanup_executors
        from waldur_core.structure.models import ResourceMixin, Service, TagMixin, VirtualMachine
        from waldur_core.structure import handlers
        from waldur_core.structure import signals as structure_signals

        from django.core import checks
        checks.register(check_cleanup_executors)

        Customer = self.get_model('Customer')
        Project = self.get_model('Project')

        CustomerPermission = self.get_model('CustomerPermission')
        ProjectPermission = self.get_model('ProjectPermission')

        signals.post_save.connect(
            handlers.log_customer_save,
            sender=Customer,
            dispatch_uid='waldur_core.structure.handlers.log_customer_save',
        )

        signals.post_delete.connect(
            handlers.log_customer_delete,
            sender=Customer,
            dispatch_uid='waldur_core.structure.handlers.log_customer_delete',
        )

        signals.post_save.connect(
            handlers.log_project_save,
            sender=Project,
            dispatch_uid='waldur_core.structure.handlers.log_project_save',
        )

        signals.post_delete.connect(
            handlers.log_project_delete,
            sender=Project,
            dispatch_uid='waldur_core.structure.handlers.log_project_delete',
        )

        # increase nc_user_count quota usage on adding user to customer
        structure_models_with_roles = (Customer, Project)
        for model in structure_models_with_roles:
            name = 'increase_customer_nc_users_quota_on_adding_user_to_%s' % model.__name__
            structure_signals.structure_role_granted.connect(
                handlers.change_customer_nc_users_quota,
                sender=model,
                dispatch_uid='waldur_core.structure.handlers.%s' % name,
            )

        # decrease nc_user_count quota usage on removing user from customer
        for model in structure_models_with_roles:
            name = 'decrease_customer_nc_users_quota_on_removing_user_from_%s' % model.__name__
            structure_signals.structure_role_revoked.connect(
                handlers.change_customer_nc_users_quota,
                sender=model,
                dispatch_uid='waldur_core.structure.handlers.%s' % name,
            )

        structure_signals.structure_role_granted.connect(
            handlers.log_customer_role_granted,
            sender=Customer,
            dispatch_uid='waldur_core.structure.handlers.log_customer_role_granted',
        )

        structure_signals.structure_role_revoked.connect(
            handlers.log_customer_role_revoked,
            sender=Customer,
            dispatch_uid='waldur_core.structure.handlers.log_customer_role_revoked',
        )

        structure_signals.structure_role_updated.connect(
            handlers.log_customer_role_updated,
            sender=CustomerPermission,
            dispatch_uid='waldur_core.structure.handlers.log_customer_role_updated',
        )

        structure_signals.structure_role_granted.connect(
            handlers.log_project_role_granted,
            sender=Project,
            dispatch_uid='waldur_core.structure.handlers.log_project_role_granted',
        )

        structure_signals.structure_role_revoked.connect(
            handlers.log_project_role_revoked,
            sender=Project,
            dispatch_uid='waldur_core.structure.handlers.log_project_role_revoked',
        )

        structure_signals.structure_role_updated.connect(
            handlers.log_project_role_updated,
            sender=ProjectPermission,
            dispatch_uid='waldur_core.structure.handlers.log_project_role_updated',
        )

        signals.pre_delete.connect(
            handlers.revoke_roles_on_project_deletion,
            sender=Project,
            dispatch_uid='waldur_core.structure.handlers.revoke_roles_on_project_deletion',
        )

        for index, model in enumerate(ResourceMixin.get_all_models()):
            signals.pre_delete.connect(
                handlers.log_resource_deleted,
                sender=model,
                dispatch_uid='waldur_core.structure.handlers.log_resource_deleted_{}_{}'.format(
                    model.__name__, index),
            )

            structure_signals.resource_imported.connect(
                handlers.log_resource_imported,
                sender=model,
                dispatch_uid='waldur_core.structure.handlers.log_resource_imported_{}_{}'.format(
                    model.__name__, index),
            )

            fsm_signals.post_transition.connect(
                handlers.log_resource_action,
                sender=model,
                dispatch_uid='waldur_core.structure.handlers.log_resource_action_{}_{}'.format(
                    model.__name__, index),
            )

            signals.post_save.connect(
                handlers.log_resource_creation_scheduled,
                sender=model,
                dispatch_uid='waldur_core.structure.handlers.log_resource_creation_scheduled_{}_{}'.format(
                    model.__name__, index),
            )

            signals.pre_delete.connect(
                handlers.delete_service_settings_on_scope_delete,
                sender=model,
                dispatch_uid='waldur_core.structure.handlers.delete_service_settings_on_scope_delete_{}_{}'.format(
                    model.__name__, index),
            )

            if issubclass(model, CoordinatesMixin):
                fsm_signals.post_transition.connect(
                    handlers.detect_vm_coordinates,
                    sender=model,
                    dispatch_uid='waldur_core.structure.handlers.detect_vm_coordinates_{}_{}'.format(
                        model.__name__, index),
                )

        for index, model in enumerate(VirtualMachine.get_all_models()):
            signals.post_save.connect(
                handlers.update_resource_start_time,
                sender=model,
                dispatch_uid='waldur_core.structure.handlers.update_resource_start_time_{}_{}'.format(
                    model.__name__, index),
            )

        signals.post_save.connect(
            handlers.connect_customer_to_shared_service_settings,
            sender=Customer,
            dispatch_uid='waldur_core.structure.handlers.connect_customer_to_shared_service_settings',
        )

        signals.post_save.connect(
            handlers.connect_project_to_all_available_services,
            sender=Project,
            dispatch_uid='waldur_core.structure.handlers.connect_project_to_all_available_services',
        )

        for index, service_model in enumerate(Service.get_all_models()):
            signals.post_save.connect(
                handlers.connect_service_to_all_projects_if_it_is_available_for_all,
                sender=service_model,
                dispatch_uid='waldur_core.structure.handlers.'
                             'connect_service_{}_to_all_projects_if_it_is_available_for_all_{}'.format(
                                 service_model.__name__, index),
            )

            signals.post_delete.connect(
                handlers.delete_service_settings_on_service_delete,
                sender=service_model,
                dispatch_uid='waldur_core.structure.handlers.delete_service_settings_on_service_delete_{}_{}'.format(
                    service_model.__name__, index),
            )

        signals.post_save.connect(
            handlers.clean_tags_cache_after_tagged_item_saved,
            sender=TagMixin.tags.through,
            dispatch_uid='waldur_core.structure.handlers.clean_tags_cache_after_tagged_item_created'
        )

        signals.pre_delete.connect(
            handlers.clean_tags_cache_before_tagged_item_deleted,
            sender=TagMixin.tags.through,
            dispatch_uid='waldur_core.structure.handlers.clean_tags_cache_after_tagged_item_created'
        )



import uuid

import six
from urlparse import urlparse

from django.contrib.contenttypes.models import ContentType
from django.urls import resolve
import django_filters
from django_filters.rest_framework import DjangoFilterBackend
from django_filters.constants import EMPTY_VALUES
from rest_framework.filters import BaseFilterBackend

from waldur_core.core import serializers as core_serializers, fields as core_fields, models as core_models


class GenericKeyFilterBackend(DjangoFilterBackend):
    """
    Backend for filtering by backend field.

    Methods 'get_related_models' and 'get_field_name' has to be implemented.
    Example:

        class AlertScopeFilterBackend(core_filters.GenericKeyFilterBackend):

            def get_related_models(self):
                return utils.get_loggable_models()

            def get_field_name(self):
                return 'scope'
    """
    content_type_field = 'content_type'
    object_id_field = 'object_id'

    def get_related_models(self):
        """ Return all models that are acceptable as filter argument """
        raise NotImplementedError

    def get_field_name(self):
        """ Get name of filter field name in request """
        raise NotImplementedError

    def get_field_value(self, request):
        field_name = self.get_field_name()
        return request.query_params.get(field_name)

    def filter_queryset(self, request, queryset, view):
        value = self.get_field_value(request)
        if value:
            field = core_serializers.GenericRelatedField(related_models=self.get_related_models())
            # Trick to set field context without serializer
            field._context = {'request': request}
            obj = field.to_internal_value(value)
            ct = ContentType.objects.get_for_model(obj)
            return queryset.filter(**{self.object_id_field: obj.id, self.content_type_field: ct})
        return queryset


class MappedFilterMixin(object):

    def __init__(self, choice_mappings, **kwargs):
        super(MappedFilterMixin, self).__init__(**kwargs)

        # TODO: enable this assert then filtering by numbers will be disabled
        # assert set(k for k, _ in self.field.choices) == set(choice_mappings.keys()), 'Choices do not match mappings'
        assert len(set(choice_mappings.values())) == len(choice_mappings), 'Mappings are not unique'

        self.mapped_to_model = choice_mappings
        self.model_to_mapped = {v: k for k, v in six.iteritems(choice_mappings)}


class MappedChoiceFilter(MappedFilterMixin, django_filters.ChoiceFilter):
    """
    A choice field that maps enum values from representation to model ones and back.

    Filter analog for MappedChoiceField.
    """

    def filter(self, qs, value):
        if value in self.mapped_to_model:
            value = self.mapped_to_model[value]
        return super(MappedChoiceFilter, self).filter(qs, value)


class MappedMultipleChoiceFilter(MappedFilterMixin, django_filters.MultipleChoiceFilter):
    """
    A multiple choice field that maps enum values from representation to model ones and back.

    Filter analog for MappedChoiceField that allow to filter by several choices.
    """

    def filter(self, qs, value):
        value = [self.mapped_to_model[v] for v in value if v in self.mapped_to_model]
        return super(MappedMultipleChoiceFilter, self).filter(qs, value)


class StateFilter(MappedMultipleChoiceFilter):

    DEFAULT_CHOICES = (
        ('Creation Scheduled', 'Creation Scheduled'),
        ('Creating', 'Creating'),
        ('Update Scheduled', 'Update Scheduled'),
        ('Updating', 'Updating'),
        ('Deletion Scheduled', 'Deletion Scheduled'),
        ('Deleting', 'Deleting'),
        ('OK', 'OK'),
        ('Erred', 'Erred'),
    )

    States = core_models.StateMixin.States
    DEFAULT_CHOICE_MAPPING = {
        'Creation Scheduled': States.CREATION_SCHEDULED,
        'Creating': States.CREATING,
        'Update Scheduled': States.UPDATE_SCHEDULED,
        'Updating': States.UPDATING,
        'Deletion Scheduled': States.DELETION_SCHEDULED,
        'Deleting': States.DELETING,
        'OK': States.OK,
        'Erred': States.ERRED,
    }

    def __init__(self, choices=DEFAULT_CHOICES, choice_mappings=None, **kwargs):
        if choice_mappings is None:
            choice_mappings = self.DEFAULT_CHOICE_MAPPING
        super(StateFilter, self).__init__(choices=choices, choice_mappings=choice_mappings, **kwargs)


class URLFilter(django_filters.CharFilter):
    """ Filter by hyperlinks. ViewSet name must be supplied in order to validate URL. """

    def __init__(self, view_name, lookup_field='uuid', **kwargs):
        super(URLFilter, self).__init__(**kwargs)
        self.view_name = view_name
        self.lookup_field = lookup_field

    def get_uuid(self, value):
        uuid_value = ''
        path = urlparse(value).path
        if path.startswith('/'):
            match = resolve(path)
            if match.url_name == self.view_name:
                uuid_value = match.kwargs.get(self.lookup_field)
        return uuid_value

    def filter(self, qs, value):
        if value in EMPTY_VALUES:
            return qs

        uuid_value = self.get_uuid(value)
        try:
            uuid.UUID(uuid_value)
        except ValueError:
            return qs.none()
        return super(URLFilter, self).filter(qs, uuid_value)


class TimestampFilter(django_filters.NumberFilter):
    """
    Filter for dates in timestamp format
    """
    def filter(self, qs, value):
        if value in EMPTY_VALUES:
            return qs

        field = core_fields.TimestampField()
        datetime_value = field.to_internal_value(value)
        return super(TimestampFilter, self).filter(qs, datetime_value)


class CategoryFilter(django_filters.CharFilter):
    """
    Filters queryset by category names.
    If category name does not match, it will work as CharFilter.

    :param categories: dictionary of category names as keys and category elements as values.
    """
    def __init__(self, categories, **kwargs):
        super(CategoryFilter, self).__init__(**kwargs)
        self.categories = categories

    def filter(self, qs, value):
        if value in self.categories.keys():
            return qs.filter(**{'%s__in' % self.name: self.categories[value]})

        return super(CategoryFilter, self).filter(qs, value)


class StaffOrUserFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        if request.user.is_staff or request.user.is_support:
            return queryset

        return queryset.filter(user=request.user)


class ContentTypeFilter(django_filters.CharFilter):

    def filter(self, qs, value):
        if value in EMPTY_VALUES:
            return qs
        try:
            app_label, model = value.split('.')
            ct = ContentType.objects.get(app_label=app_label, model=model)
            return super(ContentTypeFilter, self).filter(qs, ct)
        except (ContentType.DoesNotExist, ValueError):
            return qs.none()


class BaseExternalFilter(object):
    """ Interface for external alert filter """
    def filter(self, request, queryset, view):
        raise NotImplementedError


class ExternalFilterBackend(BaseFilterBackend):
    """
    Support external filters registered in other apps
    """

    @classmethod
    def get_registered_filters(cls):
        return getattr(cls, '_filters', [])

    @classmethod
    def register(cls, external_filter):
        assert isinstance(external_filter, BaseExternalFilter), 'Registered filter has to inherit BaseExternalFilter'
        if hasattr(cls, '_filters'):
            cls._filters.append(external_filter)
        else:
            cls._filters = [external_filter]

    def filter_queryset(self, request, queryset, view):
        for filt in self.__class__.get_registered_filters():
            queryset = filt.filter(request, queryset, view)
        return queryset


class SummaryFilter(DjangoFilterBackend):
    """ Base filter for summary querysets """

    def filter_queryset(self, request, queryset, view):
        queryset = self.filter(request, queryset, view)
        return queryset

    def get_queryset_filter(self, queryset):
        """ Return specific for queryset filter if it exists """
        raise NotImplementedError()

    def get_base_filter(self):
        """ Return base filter that could be used for all summary objects """
        raise NotImplementedError()

    def _get_filter(self, queryset):
        try:
            return self.get_queryset_filter(queryset)
        except NotImplementedError:
            return self.get_base_filter()

    def filter(self, request, queryset, view):
        """ Filter each resource separately using its own filter """
        summary_queryset = queryset
        filtered_querysets = []
        for queryset in summary_queryset.querysets:
            filter_class = self._get_filter(queryset)
            queryset = filter_class(request.query_params, queryset=queryset).qs
            filtered_querysets.append(queryset)

        summary_queryset.querysets = filtered_querysets
        return summary_queryset



from rest_framework import test, status

from waldur_core.logging.models import Alert
from waldur_core.logging.tests.factories import AlertFactory
from waldur_core.structure.models import CustomerRole
from waldur_core.structure.tests import factories


class FilterAlertsByAggregateTest(test.APITransactionTestCase):

    def setUp(self):
        self.users = []
        self.customers = []
        self.projects = []

        self.project_alerts = []
        self.customer_alerts = []

        for i in range(2):
            customer = factories.CustomerFactory()
            user = factories.UserFactory()
            customer.add_user(user, CustomerRole.OWNER)

            project = factories.ProjectFactory(customer=customer)
            resource = self.create_resource(customer, project)
            spl = resource.service_project_link
            service = spl.service

            project_alerts = (
                AlertFactory(scope=project),
                AlertFactory(scope=resource),
                AlertFactory(scope=spl)
            )

            customer_alerts = project_alerts + (
                AlertFactory(scope=service),
                AlertFactory(scope=customer)
            )

            self.users.append(user)
            self.customers.append(customer)
            self.projects.append(project)

            self.project_alerts.append(project_alerts)
            self.customer_alerts.append(customer_alerts)

        # Cleanup other alerts
        alert_ids = [alert.pk for alerts in self.customer_alerts for alert in alerts]
        Alert.objects.exclude(pk__in=alert_ids).delete()

    def test_alert_can_be_filtered_by_customer(self):
        for user, customer, alerts in zip(self.users, self.customers, self.customer_alerts):
            query = {
                'aggregate': 'customer',
                'uuid': customer.uuid.hex
            }
            self.check_alerts(user, query, alerts)

    def test_alert_can_be_filtered_by_project(self):
        for user, project, alerts in zip(self.users, self.projects, self.project_alerts):
            self.client.force_authenticate(user)
            query = {
                'aggregate': 'project',
                'uuid': project.uuid.hex
            }
            self.check_alerts(user, query, alerts)

    def check_alerts(self, user, query, alerts):
        self.client.force_authenticate(user)
        response = self.client.get(AlertFactory.get_list_url(), data=query)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        expected = set(alert.uuid.hex for alert in alerts)
        actual = set(alert['uuid'] for alert in response.data)
        self.assertEqual(expected, actual)

    def create_resource(self, customer, project):
        settings = factories.ServiceSettingsFactory(customer=customer)
        service = factories.TestServiceFactory(customer=customer, settings=settings)
        spl = factories.TestServiceProjectLinkFactory(service=service, project=project)
        resource = factories.TestNewInstanceFactory(service_project_link=spl)
        return resource



import datetime
import unittest

from rest_framework import status, test
from rest_framework.reverse import reverse

from waldur_core.core.utils import datetime_to_timestamp
from waldur_core.structure.tests.factories import TestNewInstanceFactory, TestServiceProjectLinkFactory, UserFactory

from ..models import ResourceSla, ResourceItem, ResourceSlaStateTransition
from ..utils import format_period


class BaseMonitoringTest(test.APITransactionTestCase):
    def setUp(self):
        self.link = TestServiceProjectLinkFactory()
        self.vm1 = TestNewInstanceFactory(service_project_link=self.link)
        self.vm2 = TestNewInstanceFactory(service_project_link=self.link)
        self.vm3 = TestNewInstanceFactory(service_project_link=self.link)
        self.client.force_authenticate(UserFactory(is_staff=True))


@unittest.skip('Monitoring is not supported by structure yet.')
class SlaTest(BaseMonitoringTest):
    def setUp(self):
        super(SlaTest, self).setUp()

        today = datetime.date.today()
        period = format_period(today)

        invalid_date = today + datetime.timedelta(days=100)
        invalid_period = format_period(invalid_date)

        ResourceSla.objects.create(scope=self.vm1, period=period, value=90)
        ResourceSla.objects.create(scope=self.vm1, period=invalid_period, value=70)
        ResourceSla.objects.create(scope=self.vm2, period=period, value=80)

    def test_sorting(self):
        response = self.client.get(TestNewInstanceFactory.get_list_url(), data={'o': 'actual_sla'})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(2, len(response.data))
        self.assertEqual([80, 90], [item['sla']['value'] for item in response.data])

    def test_filtering(self):
        response = self.client.get(TestNewInstanceFactory.get_list_url(), data={'actual_sla': 80})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(1, len(response.data))

    def test_actual_sla_serializer(self):
        response = self.client.get(TestNewInstanceFactory.get_url(self.vm1))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(90, response.data['sla']['value'])


@unittest.skip('Monitoring is not supported by structure yet.')
class EventsTest(BaseMonitoringTest):
    def setUp(self):
        super(EventsTest, self).setUp()

        today = datetime.date.today()
        timestamp = datetime_to_timestamp(today)
        period = format_period(today)

        ResourceSlaStateTransition.objects.create(scope=self.vm1, period=period, timestamp=timestamp, state=True)
        ResourceSlaStateTransition.objects.create(scope=self.vm2, period=period, timestamp=timestamp, state=False)

        self.url = reverse('resource-sla-state-transition-list')

    def test_scope_filter(self):
        vm1_url = TestNewInstanceFactory.get_url(self.vm1)
        response = self.client.get(self.url, data={'scope': vm1_url})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual('U', response.data[0]['state'])

        vm2_url = TestNewInstanceFactory.get_url(self.vm2)
        response = self.client.get(self.url, data={'scope': vm2_url})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual('D', response.data[0]['state'])

    def test_period_filter(self):
        url = reverse('resource-sla-state-transition-list')

        today = datetime.date.today()
        invalid_date = today + datetime.timedelta(days=100)
        invalid_period = format_period(invalid_date)

        response = self.client.get(url, data={'period': invalid_period})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(0, len(response.data))


@unittest.skip('Monitoring is not supported by structure yet.')
class ItemTest(BaseMonitoringTest):
    def setUp(self):
        super(ItemTest, self).setUp()

        ResourceItem.objects.create(scope=self.vm1, name='application_status', value=1)
        ResourceItem.objects.create(scope=self.vm2, name='application_status', value=0)

        ResourceItem.objects.create(scope=self.vm1, name='ram_usage', value=10)
        ResourceItem.objects.create(scope=self.vm2, name='ram_usage', value=20)

    def test_serializer(self):
        response = self.client.get(TestNewInstanceFactory.get_url(self.vm1))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual({'application_status': 1, 'ram_usage': 10},
                         response.data['monitoring_items'])

    def test_filter(self):
        response = self.client.get(TestNewInstanceFactory.get_list_url(),
                                   data={'monitoring__application_status': 1})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(1, len(response.data))

    def test_sorter(self):
        response = self.client.get(TestNewInstanceFactory.get_list_url(),
                                   data={'o': 'monitoring__application_status'})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        values = []
        for item in response.data:
            if not item['monitoring_items']:
                values.append(None)
            else:
                values.append(item['monitoring_items']['application_status'])
        self.assertEqual([0, 1], values)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_fsm


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0033_remove_servicesettings_dummy'),
    ]

    operations = [
        migrations.AlterField(
            model_name='servicesettings',
            name='state',
            field=django_fsm.FSMIntegerField(default=5, choices=[(5, 'Creation Scheduled'), (6, 'Creating'), (1, 'Update Scheduled'), (2, 'Updating'), (7, 'Deletion Scheduled'), (8, 'Deleting'), (3, 'OK'), (4, 'Erred')]),
        ),
    ]



# coding=utf-8
from __future__ import unicode_literals

from StringIO import StringIO

from django.core.management import call_command
from django.test import TestCase

from .. import factories


class DumpUsersCommandTest(TestCase):

    def test_dump_users_command_with_unicode_full_name(self):
        user = factories.UserFactory(full_name='äöur šipākøv')
        output = StringIO()
        try:
            call_command('dumpusers', stdout=output)
        except UnicodeDecodeError as e:
            self.fail(str(e))

        self.assertIn(user.full_name.encode('utf8'), output.getvalue())



import mock
import unittest

from waldur_core.structure.utils import update_pulled_fields


class InstanceMock(object):
    def __init__(self, name='Virtual machine', runtime_state='OK', error_message=''):
        self.pk = 1
        self.name = name
        self.runtime_state = runtime_state
        self.error_message = error_message
        self.save = mock.Mock()


class UpdatePulledFieldsTest(unittest.TestCase):
    def test_model_is_not_saved_if_pulled_fields_are_the_same(self):
        vm = InstanceMock()
        update_pulled_fields(vm, vm, ('name', 'runtime_state'))
        self.assertEqual(vm.save.call_count, 0)

    def test_model_is_saved_if_pulled_fields_are_different(self):
        vm1 = InstanceMock()
        vm2 = InstanceMock(runtime_state='ERRED')
        update_pulled_fields(vm1, vm2, ('name', 'runtime_state'))
        self.assertEqual(vm1.save.call_count, 1)

    def test_model_is_not_saved_if_changed_fields_are_ignored(self):
        vm1 = InstanceMock()
        vm2 = InstanceMock(runtime_state='ERRED')
        update_pulled_fields(vm1, vm2, ('name',))
        self.assertEqual(vm1.save.call_count, 0)

    def test_error_message_saved_if_it_changed(self):
        vm1 = InstanceMock()
        vm2 = InstanceMock(error_message='Server does not respond.')
        update_pulled_fields(vm1, vm2, ('name',))
        self.assertEqual(vm1.save.call_count, 1)



from __future__ import unicode_literals

import copy

from django.conf import settings
from django.test.utils import override_settings
from rest_framework import test, status


class PermissionsTest(test.APITransactionTestCase):
    """
    Abstract class for permissions tests.

    Methods `get_urls_configs`, `get_users_with_permission`,
    `get_users_without_permissions` have to be overridden.

    Logical example:

    class ExamplePermissionsTest(PermissionsTest):

        def get_users_with_permission(self, url, method):
            if is_unreachable(url):
                # no one can has access to unreachable url
                return []
            return [user_with_permission]

        def get_users_without_permissions(self, url, method):
            if is_unreachable(url):
                # everybody does not have access to to unreachable url
                return [user_with_permission, user_without_permission]
            return [user_without_permission]

        def get_urls_configs(self):
            yield {'url': 'http://testserver/some/url, 'method': 'GET'}
            yield {'url': 'http://testserver/some/unreachable/url', 'method': 'POST'}
            ...
    """

    def get_urls_configs(self):
        """
        Return list or generator of url configs.

        Each url config is dictionary with such keys:
         - url: url itself
         - method: request method
         - data: data which will be sent in request
        url config example:
        {
            'url': 'http://testserver/api/backup/',
            'method': 'POST',
            'data': {'backup_source': 'backup/source/url'}
        }
        """
        raise NotImplementedError()

    def get_users_with_permission(self, url, method):
        """
        Return list of users which can access given url with given method
        """
        raise NotImplementedError()

    def get_users_without_permissions(self, url, method):
        """
        Return list of users which can not access given url with given method
        """
        raise NotImplementedError()

    def test_permissions(self):
        """
        Go through all url configs ands checks that user with permissions
        can request them and users without - can't
        """
        for conf in self.get_urls_configs():
            url, method = conf['url'], conf['method']
            data = conf['data'] if 'data' in conf else {}

            for user in self.get_users_with_permission(url, method):
                self.client.force_authenticate(user=user)
                response = getattr(self.client, method.lower())(url, data=data)
                self.assertFalse(
                    response.status_code in (status.HTTP_403_FORBIDDEN, status.HTTP_404_NOT_FOUND),
                    'Error. User %s can not reach url: %s (method:%s). (Response status code %s, data %s)'
                    % (user, url, method, response.status_code, response.data))

            for user in self.get_users_without_permissions(url, method):
                self.client.force_authenticate(user=user)
                response = getattr(self.client, method.lower())(url, data=data)
                unreachable_statuses = (status.HTTP_403_FORBIDDEN, status.HTTP_404_NOT_FOUND, status.HTTP_409_CONFLICT)
                self.assertTrue(
                    response.status_code in unreachable_statuses,
                    'Error. User %s can reach url: %s (method:%s). (Response status code %s, data %s)'
                    % (user, url, method, response.status_code, response.data))


class ListPermissionsTest(test.APITransactionTestCase):
    """
    Abstract class that tests what objects user receive in list.

    Method `get_users_and_expected_results` has to be overridden.
    Method `get_url` have to be defined.
    """
    def get_url(self):
        return None

    def get_users_and_expected_results(self):
        """
        Return list or generator of dictionaries with such keys:
         - user - user which we want to test
         - expected_results - list of dictionaries with fields which user has
                              to receive as answer from server
        """
        pass

    def test_list_permissions(self):
        for user_and_expected_result in self.get_users_and_expected_results():
            user = user_and_expected_result['user']
            expected_results = user_and_expected_result['expected_results']

            self.client.force_authenticate(user=user)
            response = self.client.get(self.get_url())
            self.assertEqual(
                len(expected_results), len(response.data),
                'User %s receive wrong number of objects. Expected: %s, received %s'
                % (user, len(expected_results), len(response.data)))
            for actual, expected in zip(response.data, expected_results):
                for key, value in expected.iteritems():
                    self.assertEqual(actual[key], value)


def override_waldur_core_settings(**kwargs):
    waldur_settings = copy.deepcopy(settings.WALDUR_CORE)
    waldur_settings.update(kwargs)
    return override_settings(WALDUR_CORE=waldur_settings)
